<?php

namespace Drupal\brevo_contact_sync\Form;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\sendinblue_api\Service\SendinblueApi;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the Users Field Mapping Form.
 */
class BrevoContactMappingForm extends ConfigFormBase {

  /**
   * Drupal cache.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Sendinblue API service.
   *
   * @var \Drupal\sendinblue_api\Service\SendinblueApi
   */
  protected $sendinblueApi;

  /**
   * Entity Field Manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->setCacheManager($container->get('cache.default'));
    $instance->setSendinblueApi($container->get('sendinblue_api'));
    $instance->setEntityFieldManager($container->get('entity_field.manager'));
    return $instance;
  }

  /**
   * Sets the cache manager dependency.
   *
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache service.
   */
  protected function setCacheManager(CacheBackendInterface $cache) {
    $this->cache = $cache;
  }

  /**
   * Sets the Sendinblue API dependency.
   *
   * @param \Drupal\sendinblue_api\Service\SendinblueApi $sendinblueApi
   *   The Sendinblue API service.
   */
  protected function setSendinblueApi(SendinblueApi $sendinblueApi) {
    $this->sendinblueApi = $sendinblueApi;
  }

  /**
   * Sets the Entity Field Manager dependency.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager service.
   */
  protected function setEntityFieldManager(EntityFieldManagerInterface $entityFieldManager) {
    $this->entityFieldManager = $entityFieldManager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'user_mapping_config_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['brevo_contact_sync.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->sendinblueApi->getConfig();
    if (empty($config['api_key'])) {
      $form['markup'] = [
        '#markup' => $this->t('You must authorize Sendinblue API before choose mapping or there are no lists available on your account.'),
      ];
      return $form;
    }
    $mappingFields = [];
    $lists = $this->sendinblueApi->getMailingLists();
    $custom_fields_data = $this->sendinblueApi->getCustomFields(FALSE);
    $custom_fields = !is_null($custom_fields_data) ? $custom_fields_data->custom_fields : FALSE;
    foreach ($custom_fields as $values) {
      $mappingFields[$values->name] = $values->name;
    }

    $options = [];
    if (!empty($lists)) {
      foreach ($lists as $list) {
        $options[$list->list_id] = $list->name;
      }
    }

    $usersFields = [];
    $listUsersFields = $this->entityFieldManager->getFieldDefinitions('user', 'user');
    $staticFields = ['name', 'uid', 'mail', 'status', 'roles'];
    foreach ($listUsersFields as $field) {
      $fieldName = $field->getName();
      if ((strpos($fieldName, 'field_') === 0) || in_array($fieldName, $staticFields)) {
        $label = $field->getLabel();
        $usersFields[$fieldName] = is_object($label) ? $label->__toString() : (string) $label;
      }
    }

    $config = $this->config('brevo_contact_sync.settings');
    $mappings = $form_state->get('mappings');
    $selected_list = $config->get('selected_list');
    if ($mappings === NULL) {
      $mappings = $config->get('mappings') ?? [
        ['user_field' => '', 'mapping_field' => '']
      ];
      $form_state->set('mappings', $mappings);
    }
    $mappingValue = ['key' => 'Key', 'value' => 'Value'];
    $form['mappings'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('User Field'),
        $this->t('Value'),
        $this->t('Mapping Field'),
        $this->t('Operations'),
      ],
      '#prefix' => '<div id="mappings-wrapper">',
      '#suffix' => '</div>',
    ];

    foreach ($mappings as $index => $mapping) {
      $form['lists'] = [
        '#type' => 'select',
        '#title' => $this->t('List'),
        '#required' => TRUE,
        '#options' => $options,
        '#default_value' => $selected_list,
        '#empty_option' => $this->t('- Select a list -'),
        '#weight' => -10,
      ];
      $form['mappings'][$index]['user_field'] = [
        '#type' => 'select',
        '#options' => $usersFields,
        '#default_value' => $mapping['user_field'],
        '#title' => $this->t('User Field'),
      ];
      $form['mappings'][$index]['ref_value'] = [
        '#type' => 'select',
        '#options' => ['' => $this->t('Select value')] + $mappingValue,
        '#default_value' => $mapping['ref_value'] ?? '',
        '#title' => $this->t('Value'),
      ];
      $form['mappings'][$index]['mapping_field'] = [
        '#type' => 'select',
        '#options' => $mappingFields,
        '#default_value' => $mapping['mapping_field'],
        '#title' => $this->t('Mapping Field'),
      ];
      $form['mappings'][$index]['remove'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#submit' => ['::removeMapping'],
        '#name' => 'remove-' . $index,
        '#ajax' => [
          'callback' => '::ajaxCallback',
          'wrapper' => 'mappings-wrapper',
        ],
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add Mapping'),
      '#submit' => ['::addMapping'],
      '#ajax' => [
        'callback' => '::ajaxCallback',
        'wrapper' => 'mappings-wrapper',
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * AJAX callback to refresh the mappings table.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    return $form['mappings'];
  }

  /**
   * Submit handler for adding a new mapping.
   */
  public function addMapping(array &$form, FormStateInterface $form_state) {
    $mappings = $form_state->get('mappings') ?? [];
    $mappings[] = ['user_field' => '', 'mapping_field' => ''];
    $form_state->set('mappings', $mappings);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Submit handler for removing a mapping.
   */
  public function removeMapping(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $index = substr($triggering_element['#name'], 7);
    $mappings = $form_state->get('mappings');
    unset($mappings[$index]);
    $form_state->set('mappings', array_values($mappings));
    $form_state->setRebuild(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $selected_list = $form_state->getValue('lists');
    $this->config('brevo_contact_sync.settings')
      ->set('mappings', $form_state->getValue('mappings'))
      ->set('selected_list', $selected_list)
      ->save();
    parent::submitForm($form, $form_state);
  }

}
